<!---+
	Service that deals with routing requests and generating urls.
--->
<cfcomponent output="false" extends="AbstractService">
	
	<cfset mixin("objectcache")>
	
	<!---+
		Loads routes in from a config file. Searches the application bundle for
		/config/Routes.cfm or /config/Routes.xml or loads the default inside
		accelerate.
	--->
	<cffunction name="loadRoutes" access="public" returntype="any" output="false">
		<cfset var routePath = getFramework().getApplicationRoot() & "/config/Routes">
		<cfset var beans = "">
		<cfset var name = "">
		<cfset var filter = "">
		
		<!--- Framework defaults --->
		<cfset var defaults = {
			controller=getConfig().getDefaultController(),
			action="OnActionMissing",
			id=0,
			format="html"
		}>
		
		<!---
			Load in routes in steps:
			
			1. If RouteConfigPath exists then load it.
			2. If {appRoot}/config/Routes.cfm exists then load it.
			3. If {appRoot}/config/Routes.xml exists then load it.
			4. Load default routes from framework.
		--->
		<cfif len(getConfig().getRouteConfigPath())>
			<cfset getRoutingEngine().loadRoutes(getConfig().getRouteConfigPath())>

		<cfelseif fileExists(routePath & ".cfm")>
			<cfset getRoutingEngine().loadRoutes(getMapping() & "/config/Routes.cfm")>

		<cfelseif fileExists(routePath & ".xml")>
			<cfset getRoutingEngine().loadRoutes(routePath & ".xml")>

		<cfelse>
			<cfset getRoutingEngine().loadRoutes(
				getDirectoryFromPath(getCurrentTemplatePath()) & "/../../config/Routes.xml"
			)>
		</cfif>
		
		<cfset structAppend(getMapper().getRoutingDefaults(),defaults,false)>
		
		<!--- Emergency fall back route 
			
			Use __PATH__, because the Mapper already provides that to us.
		--->
		<cfset getMapper().connect(
			path="/*__PATH__",
			params=struct( __PATH__=struct( required="false" ) )
		)>
		
		<cfset setMaxCachedObjects(getMapper().getMaxCachedRoutes())>
		<cfset setObjectCacheLockName("RoutingService_#createUUID()#")>
	</cffunction>
	
	<!---+
		Gets a url from the specified parameters by looking at the configured routes.
	--->
	<cffunction name="urlFor" output="false" returntype="string">
		<cfargument name="controller" type="string" required="false">
		<cfargument name="action" type="string" required="false">
		<cfargument name="_appendSlash" type="boolean" required="false">
		<cfargument name="_scriptName" type="string" required="false">
		<cfargument name="_absoluteUrl" type="boolean" required="false">
		<cfargument name="_protocol" type="string" required="false">
		<cfargument name="_serverName" type="string" required="false">
		<cfargument name="_name" type="string" required="false" default="">
		<cfargument name="_relative" type="boolean" required="false" default="false">
		<cfargument name="_pathOnly" type="boolean" required="false" default="false">
		
		<cfset var params = structNew()>
		<cfset var param = "">
		<cfset var argsString = "">
		<cfset var context = getRequestContext()>
		<cfset var result = "">
				
		<cfif arguments._relative and not isBoolean(context.getRoute())>
			<cfset structAppend( arguments, context.getRoute(), false )>
			<cfset arguments._relative = false>
		</cfif>
		
		<cfif arguments._pathOnly>
			<cfset arguments._scriptName = "">
			<cfset arguments._absoluteUrl = false>
			<cfset arguments._pathOnly = false>
		</cfif>
		
		<cfset argsString = arguments.toString()>
		<cfset result = getCachedObject(argsString)>
		
		<cfif isBoolean(result)>
			<cfset params.searchParams = structNew()>
			
			<cfloop collection="#arguments#" item="param">
				<cfif structKeyExists(arguments,param)>
					<cfif left(param,1) eq "_">
						<cfset params[right(param,len(param)-1)] = arguments[param]>
					<cfelse>
						<cfset params.searchParams[param] = lCase(arguments[param])>
					</cfif>
				</cfif>
			</cfloop>
			
			<cfset result = getMapper().generate( argumentCollection=params )>
			<cfset addCachedObject(argsString,result)>
		</cfif>
		
		<cfreturn result>
	</cffunction>
	
	<!---+
		Gets the mapper from the routing engine.
	--->
	<cffunction name="getMapper" access="public" returntype="any" output="false">

		<cfreturn getRoutingEngine().getMapper()>
	</cffunction>

	<cffunction name="setRoutingEngine" access="public" returntype="void" output="false">
		<cfargument name="value" type="any" required="true">

		<cfset variables.routingEngine = arguments.value>
	</cffunction>
	
	<cffunction name="getRoutingEngine" access="public" returntype="any" output="false">

		<cfreturn variables.routingEngine>
	</cffunction>

</cfcomponent>