<!---+
	Core of the Shared Application Platform. Provides access to the entire system.
--->
<cfcomponent output="false">
		
	<cffunction name="init" access="public" returntype="any" output="false">
		<cfargument name="applicationName" type="string" required="true">
		<cfargument name="rootPath" type="string" required="true">
		
		<cfset var serviceLoader = "">
		<cfset var appConfigPath = expandPath("#rootPath#/config/coldspring/config.xml")>
		<cfset var coldSpring = createObject("component","coldspring.beans.DefaultXmlBeanFactory").init()>

		<cfset coldSpring.loadBeansFromXmlFile(expandPath("/saa/config/coldspring/config.xml"))>
		<cfset serviceLoader = createObject("component","saa.framework.coldspring.ServiceLoader").init()>
		<cfset serviceLoader.setColdSpring(coldSpring)>
		<cfset serviceLoader.loadServices("/saa/services")>
				
		<cfset serviceLoader.loadServices("#rootPath#/services/")>

		<cfset variables.objectCache = structNew()>
		<cfset variables.applicationName = arguments.applicationName>
		<cfset variables.rootPath = arguments.rootPath>
		<cfset variables.coldSpring = serviceLoader.getColdSpring()>
		<cfset variables.subApplications = structNew()>
		<cfset variables.requestContext = createObject("java","java.lang.ThreadLocal").init()>
		
		<cfif fileExists(appConfigPath)>
			<cfset getColdSpring().loadBeansFromXmlFile(appConfigPath)>
		</cfif>
		
		<cfset autowireServices(getColdSpring())>
		
		<cfreturn this>
	</cffunction>
				
	<cffunction name="getObject" access="public" returntype="any" output="false">
		<cfargument name="name" type="string" required="true">
		
		<cfset var cs = "">
		<cfset var beanDef = "">
		<cfset var object = "">
		
		<cfif structKeyExists(variables.objectCache,arguments.name)>
			<cfreturn variables.objectCache[arguments.name]>
		</cfif>
		
		<cfset cs = getColdSpring()>
		<cfset beanDef = cs.getBeanDefinition(arguments.name)>
		<cfset object = cs.getBean(arguments.name)>
		
		<cfif beanDef.isSingleton()>
			<cfset variables.objectCache[arguments.name] = object>
		</cfif>
		
		<cfreturn object>
	</cffunction>
			
	<cffunction name="getSubApplications" access="public" returntype="struct" output="false">

		<cfset var result = "">

		<cflock name="saa.beans.ApplicationCore(#getApplicationName()#)" timeout="10">
			<cfset result = structCopy(variables.subApplications)>
		</cflock>
		
		<cfreturn result>
	</cffunction>
	
	<cffunction name="getSubApplication" access="public" returntype="any" output="false">
		<cfargument name="name" type="string" required="true">
		
		<cfreturn variables.subApplications[name]>
	</cffunction>

	<cffunction name="registerSubApplication" access="public" returntype="void" output="false">
		<cfargument name="name" type="string" required="true">
		<cfargument name="context" type="any" required="true">

		<cflock name="saa.beans.ApplicationCore(#getApplicationName()#)" timeout="10">
			<cfset variables.subApplications[name] = context>
		</cflock>
	</cffunction>
	
	<cffunction name="unregisterSubApplication" access="public" returntype="void" output="false">
		<cfargument name="name" type="string" required="true">
		
		<cflock name="saa.beans.ApplicationCore(#getApplicationName()#)" timeout="10">
			<cfset structDelete(variables.subApplications,name)>
		</cflock>
	</cffunction>
	
	<!---+
		Get a scope by name.
		
		Convenience method for getScopes().getScope(name).
	--->
	<cffunction name="getScope" access="public" returntype="any" output="false">
		<cfargument name="name" type="string" required="true">
		
		<cfreturn getScopes().getScope(arguments.name)>
	</cffunction>
	
	<cffunction name="getScopes" access="public" returntype="any" output="false">
		
		<cfreturn getObject("Shared.Services.Scoping").getScopes()>
	</cffunction>
		
	<cffunction name="getConfig" access="public" returntype="any" output="false">

		<cfreturn getObject("Shared.Application.Config")>
	</cffunction>
			
	<cffunction name="getColdSpring" access="public" returntype="any" output="false">

		<cfreturn variables.coldSpring>
	</cffunction>
		
	<cffunction name="getApplicationName" access="public" returntype="any" output="false">

		<cfreturn variables.applicationName>
	</cffunction>
	
	<cffunction name="getRootPath" access="public" returntype="any" output="false">

		<cfreturn variables.rootPath>
	</cffunction>
	
	<!---+
		Returns a request specific context.
	--->
	<cffunction name="getRequestContext" access="public" returntype="struct" output="false">
		<cftry>
			<cfreturn request.application>
		<cfcatch type="any">
			<!--- Default is there's no context (ie. we're in an async CF event) --->
			<cfreturn variables.requestContext.get()>
		</cfcatch>
		</cftry>
	</cffunction>
	
	<!---+
		Gets the current subapplication name or "ColdFusion.AsyncApplicationEvent"
		if this is an async event like onSessionEnd().
	--->
	<cffunction name="getCurrentSubApplicationName" access="public" returntype="string" output="false">

		<cfreturn getRequestContext().getSubApplicationName()>
	</cffunction>
	
	<!---+
		Returns an application specific struct.
	--->
	<cffunction name="getCurrentSubApplicationContext" access="public" returntype="any" output="false">

		<cfreturn getSubApplication(getCurrentSubApplicationName())>
	</cffunction>
	
	<!---+
		Dispatches an application event, such as "onApplicationStart" originating
		from the specified request.
		
		Special application.cfc methods should call this with the method name and 
		the arguments structure to broadcast the event to all services which are 
		setup to listen for it.
	--->
	<cffunction name="dispatchEvent" access="public" returntype="void" output="false">
		<cfargument name="requestContext" type="any" required="true">
		<cfargument name="eventName" type="string" required="true">
		<cfargument name="args" type="struct" required="true">
		
		<cfset variables.requestContext.set(arguments.requestContext)>
		
		<cftry>
			<cfset getObject("Shared.Services.ApplicationEvent")
				.dispatchEvent(arguments.eventName, arguments.args)>
			
			<cfcatch type="any">
				<cfset variables.requestContext.set(javaCast("null",""))>
				<cfrethrow>
			</cfcatch>
		</cftry>
		
		<cfset variables.requestContext.set(javaCast("null",""))>
	</cffunction>
	
	<!---+
		Checks for CS beans defined with the prefix "Shared.Services." and autowires them
		with a reference to the app core.
	--->
	<cffunction name="autowireServices" access="private" returntype="void" output="false">
		<cfargument name="coldSpring" type="any" required="true">
		<cfargument name="visited" type="any" required="false" default="#structNew()#">
		
		<cfset var beans = arguments.coldSpring.getBeanDefinitionList()>
		<cfset var beanName = "">
		<cfset var service = "">
		<cfset var parentBeanFactory = "">
				
		<cfloop collection="#beans#" item="beanName">
			<cfif left(beanName,16) eq "Shared.Services." and not structKeyExists(arguments.visited,beanName)>
				<cfset service = beans[beanName].getBeanInstance()>
				<cfif structKeyExists(service,"setApplication") and isCustomFunction(service.setApplication)>
					<cfset service.setApplication(this)>
				</cfif>
				<cfset arguments.visited[beanName] = true>
			</cfif>
		</cfloop>
		
		<cfset parentBeanFactory = arguments.coldspring.getParent()>
		<cfif getMetaData(parentBeanFactory).name neq "coldspring.beans.AbstractBeanFactory">
			<cfset autowireServices(parentBeanFactory,visited)>
		</cfif>
		
		<!--- Starts up services so they're not lazy. --->
		<cfloop collection="#visited#" item="service">
			<cfset getObject(service)>
		</cfloop>
	</cffunction>
	
</cfcomponent>