<cfcomponent output="false">
	
	<cffunction name="init" access="public" returntype="any" output="false">
		<cfargument name="worksheetService" type="any" required="true">
		<cfargument name="eventId" type="numeric" required="true">
		<cfargument name="name" type="string" required="true">

		<cfset variables.records = queryNew("")>
		
		<cfset variables.eventId = arguments.eventId>
		<cfset variables.name = arguments.name>
		<cfset variables.worksheetService = arguments.worksheetService>
		
		<cfreturn this>
	</cffunction>
	
	<cffunction name="getEventId" access="public" returntype="numeric" output="false">

		<cfreturn variables.eventId>
	</cffunction>
	
	<cffunction name="getName" access="public" returntype="string" output="false">

		<cfreturn variables.name>
	</cffunction>
	
	<cffunction name="getFileName" access="public" returntype="string" output="false">
		
		<cfreturn "#variables.worksheetService.getWorksheetDirectory()#/#getEventId()#/#getName()#.xml">
	</cffunction>
	
	<cffunction name="getRecords" access="public" returntype="query" output="false">

		<cfreturn variables.records>
	</cffunction>
	
	<cffunction name="vote" access="public" returntype="boolean" output="false">
		<cfargument name="topicId" type="any" required="true">
		
		<cfset var record = "">
		
		<cfquery name="record" dbtype="query">
			select *
			from variables.records
			where topicId = #val(arguments.topicId)#
		</cfquery>
		
		<cfif record.recordCount>
			<cfset setValue(arguments.topicId,"vote",not record.vote)>
			<cfreturn true>
		</cfif>
		
		<cfreturn false>
	</cffunction>
	
	<cffunction name="getTopic" access="public" returntype="struct" output="false">
		<cfargument name="topicId" type="numeric" required="true">

		<cfset var topic = "">
		<cfset var result = structNew()>
		<cfset var column = "">

		<cfquery name="topic" dbtype="query">
			select *
			from variables.records
			where
				topicId = #val(arguments.topicId)#
		</cfquery>
		
		<cfif topic.recordCount>
			<cfloop list="#topic.columnList#" index="column">
				<cfset result[lCase(column)] = topic[column][topic.currentRow]>
			</cfloop>
		</cfif>
		
		<cfreturn result>
	</cffunction>
	
	<cffunction name="setTrack" access="remote" returntype="boolean" output="false">
		<cfargument name="topicId" type="numeric" required="true">
		<cfargument name="trackId" type="numeric" required="true">

		<cfset var tracks = "">
		
		<cfquery name="tracks" datasource="#variables.worksheetService.getDatasource()#">
			select trackId, trackName
			from conftrack tr
			where
				tr.eventId = <cfqueryparam value="#variables.eventId#">
		</cfquery>

		<cfquery name="tracks" dbtype="query">
			select trackId, trackName
			from tracks
			where trackId = #val(arguments.trackId)#
		</cfquery>

		<cfif tracks.recordCount>
			<cfset setValue(arguments.topicId,"trackId",tracks.trackId)>
			
			<cfreturn true>
		</cfif>

		<cfreturn false>
	</cffunction>
	
	<cffunction name="setValue" access="public" returntype="void" output="false">
		<cfargument name="topicId" type="any" required="true">
		<cfargument name="column" type="any" required="true">
		<cfargument name="value" type="any" required="true">

		<cfloop query="variables.records">
			<cfif variables.records.topicId eq arguments.topicId>
				<cfset querySetCell(variables.records,arguments.column,arguments.value,variables.records.currentRow)>
				<cfbreak>
			</cfif>
		</cfloop>
	</cffunction>
	
	<cffunction name="save" access="public" returntype="any" output="false">
		<cfset var output = "">
		<cfset var savableRecords = getRecords()>
		
		<cfquery name="savableRecords" dbtype="query">
			select
				topicId,
				trackId,
				originalTrackId,
				vote
			from
				savableRecords
		</cfquery>
		
		<cfwddx action="cfml2wddx" input="#savableRecords#" output="output">
		
		<cftry>
			<cfdirectory action="create" directory="#getDirectoryFromPath(getFileName())#">
		<cfcatch type="any">
		</cfcatch>
		</cftry>
		
		<cflock name="topicworksheet.Worksheet(#getFileName()#)" timeout="30">
 			<cfif len(getName())>
				<cffile action="write" file="#getFileName()#" output="#output#">
				<cfset variables.worksheetService.notifyWorksheetChanged(this)>
			</cfif>
		</cflock>
		
		<cfreturn this>
	</cffunction>
	
	<cffunction name="load" access="public" returntype="any" output="false">
		<cfset var topics = "">
		<cfset var worksheet = "">
		<cfset var wddx = "">
		<cfset var topicsValues = "">
		
		<cfquery name="topics" datasource="#variables.worksheetService.getDatasource()#" cachedwithin="#createTimeSpan(0,0,5,0)#">
			select
				topicId, 
				topicName, 
				t.trackId, 
				t.trackId as originalTrackId,
				shortdesc as topicDescription,
				s.slnam as speakerFirstName, 
				s.sfnam as speakerLastName, 
				s.email as speakerEmail, 
				suggester_name as suggesterName, 
				suggester_email as suggesterEmail, 
				suggester_comments as suggesterComments,
				0 as vote
			from conftopic t
			inner join conftrack tr on t.trackId = tr.trackId
			inner join cfconf_speakers s on s.speakerid = t.speakerId
			where t.eventId = <cfqueryparam value="#variables.eventId#">
			order by 
				t.topicId desc
		</cfquery>
				
		<cflock name="topicworksheet.Worksheet(#getFileName()#)" timeout="30">
			<cfif fileExists(getFileName()) and len(getName())>
				<cffile action="read" file="#getFileName()#" variable="wddx">
			</cfif>
		</cflock>
		
		<cfif len(wddx)>
			<cfwddx action="wddx2cfml" input="#wddx#" output="worksheet">

			<cfquery name="topicsValues" dbtype="query">
				select 
					max(topicId) as maxTopicId 
				from 
					topics
			</cfquery>
			
			<cfquery name="variables.records" dbtype="query">
				select 
					topics.topicId,
					topics.topicName,
					worksheet.trackId,
					topics.originalTrackId,
					topics.topicDescription,
					topics.speakerFirstName, 
					topics.speakerLastName, 
					topics.speakerEmail, 
					topics.suggesterName, 
					topics.suggesterEmail, 
					topics.suggesterComments,
					worksheet.vote
				from
					worksheet, topics
				where
					worksheet.topicId = topics.topicId
				union
				select
					topics.*
				from
					topics
				where
					topics.topicId > #val(topicsValues.maxTopicId)#
				order by 
					topicId desc
			</cfquery>
		<cfelse>
			<cfset variables.records = topics>
		</cfif>
		
		<cfreturn this>
	</cffunction>

</cfcomponent>